@@ -6,7 +6,7 @@
  * Descr.: FD to C macros - header file                                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -50,7 +50,7 @@
  * Descr.: parameter header file                                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -95,6 +95,8 @@
 /* NB: if NB_OF_X_REGS is changed it is necessary to modify ma2asm but
    also the byte code management */
 
+#define PROLOG_FILE_SUFFIX         ".pl"
+#define PROLOG_FILE_SUFFIXES_ALT   "|.pro|.prolog|"
 /*-------------------------------------------------------------------------*
  * GNU Prolog                                                              *
  *                                                                         *
@@ -103,7 +105,7 @@
  * Descr.: Wam architecture definition - description file                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -140,7 +142,7 @@
  * Descr.: Wam long type definition                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -183,7 +185,7 @@
  * Descr.: general configuration file (handled by autoconf) - header file  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -314,14 +316,17 @@
 /* Define if the cpu is a sparc */
 /* #undef M_sparc */
 
+/* Define if the cpu is a sparc64 */
+/* #undef M_sparc64 */
+
 /* Define if the cpu is a ix86 */
-#define M_ix86 1
+/* #undef M_ix86 */
 
 /* Define if the cpu is a powerpc */
 /* #undef M_powerpc */
 
 /* Define if the cpu is a x86-64 */
-/* #undef M_x86_64 */
+#define M_x86_64 1
 
 
 /* Define if the OS is an SGI IRIX */
@@ -374,7 +379,7 @@
 /* #undef M_ix86_sco */
 
 /* Define if the system is a ix86/freebsd or openbsd or netbsd */
-#define M_ix86_bsd 1
+/* #undef M_ix86_bsd */
 
 /* Define if the system is a ix86/cygwin */
 /* #undef M_ix86_cygwin */
@@ -400,6 +405,9 @@
 /* Define if the system is a sparc/bsd */
 /* #undef M_sparc_bsd */
 
+/* Define if the system is a sparc64/bsd */
+/* #undef M_sparc64_bsd */
+
 /* Define if the system is a powerpc/linux */
 /* #undef M_powerpc_linux */
 
@@ -416,7 +424,7 @@
 /* #undef M_x86_64_solaris */
 
 /* Define if the system is a x86-64/bsd */
-/* #undef M_x86_64_bsd */
+#define M_x86_64_bsd 1
 
 /* Define if the system is a x86-64/mingw */
 /* #undef M_x86_64_mingw */
@@ -445,9 +453,9 @@
 
 #define PROLOG_DIALECT "gprolog"
 #define PROLOG_NAME "GNU Prolog"
-#define PROLOG_VERSION "1.4.4"
-#define PROLOG_DATE "Oct 11 2018"
-#define PROLOG_COPYRIGHT "Copyright (C) 1999-2013 Daniel Diaz"
+#define PROLOG_VERSION "1.4.5"
+#define PROLOG_DATE "Sep 26 2021"
+#define PROLOG_COPYRIGHT "Copyright (C) 1999-2021 Daniel Diaz"
 
 #define TOP_LEVEL "gprolog"
 #define GPLC "gplc"
@@ -455,13 +463,13 @@
 #define ENV_VARIABLE "PL_PATH"
 
 #define M_VENDOR "unknown"
-#define M_CPU "i386"
-#define M_OS "openbsd6.4"
+#define M_CPU "x86_64"
+#define M_OS "openbsd7.0"
 
 #define CC "cc"
 #define CFLAGS_PREFIX_REG "-ffixed-%s"
 #define CFLAGS "-O2 -pipe  "
-#define CFLAGS_MACHINE "-march=i486 -m32 -fno-strict-aliasing"
+#define CFLAGS_MACHINE " -fno-strict-aliasing"
 #define LDFLAGS ""
 #define LDLIBS "-lm"
 #define AS "as"
@@ -485,8 +493,8 @@
 #define LIB_BIPS_FD "libbips_fd.a"
 
 
-#define SIZEOF_LONG 4
-#define SIZEOF_VOIDP 4
+#define SIZEOF_LONG 8
+#define SIZEOF_VOIDP 8
 
 #define SIZEOF_PTR                 SIZEOF_VOIDP
 #define WORD_SIZE                  (8 * SIZEOF_PTR)
@@ -508,7 +516,7 @@
  * Descr.: architecture dependent features - Header file                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -734,7 +742,7 @@
 #define siglongjmp longjmp
 #endif
 
-#if defined(_WIN64) && !defined(_MSC_VER)
+#if defined(_WIN64) && !defined(_MSC_VER) && !defined(__CYGWIN__)
 /* Mingw64-gcc implements setjmp with msvcrt's _setjmp. This _setjmp
  * has an additional (hidden) argument. If it is NULL, longjmp will NOT do
  * stack unwinding (needed for SEH). By default the the second argument is
@@ -1000,42 +1008,52 @@ typedef WamWord *WamWordP;
 
    /*--- Begin Register Generation ---*/
 
-#define TR			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+0])
-#define B			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+1])
-#define H			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+2])
-#define HB1			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+3])
-#define CP			(((WamCont  *) pl_reg_bank)[NB_OF_X_REGS+4])
-#define E			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+5])
-#define CS			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+6])
-#define S			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+7])
-#define STAMP			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+8])
-#define BCI			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+9])
-#define LSSA			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+10])
+register WamWord 		*pl_reg_bank asm ("r12");
+
+register WamWordP		TR  asm ("r13");
+register WamWordP		B   asm ("r14");
+register WamWordP		H   asm ("r15");
+
+
+#define HB1			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+0])
+#define CP			(((WamCont  *) pl_reg_bank)[NB_OF_X_REGS+1])
+#define E			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+2])
+#define CS			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+3])
+#define S			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+4])
+#define STAMP			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+5])
+#define BCI			(((WamWord  *) pl_reg_bank)[NB_OF_X_REGS+6])
+#define LSSA			(((WamWordP *) pl_reg_bank)[NB_OF_X_REGS+7])
 
 
 #define NB_OF_REGS          	11
-#define NB_OF_ALLOC_REGS    	0
-#define NB_OF_NOT_ALLOC_REGS	11
+#define NB_OF_ALLOC_REGS    	3
+#define NB_OF_NOT_ALLOC_REGS	8
 #define REG_BANK_SIZE       	(NB_OF_X_REGS+NB_OF_NOT_ALLOC_REGS)
 
 
 
 
-#define NB_OF_USED_MACHINE_REGS 0
+#define NB_OF_USED_MACHINE_REGS 4
 
 #ifdef ENGINE_FILE
 
-WamWord pl_reg_bank[REG_BANK_SIZE];
+WamWord *save_reg_bank;
+
+WamWord pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS + 1];
+
 char *pl_reg_tbl[] = { "TR", "B", "H", "HB1", "CP", "E", "CS", "S", "STAMP", "BCI", "LSSA"};
 
 #else
 
-extern WamWord pl_reg_bank[];
+extern WamWord *save_reg_bank;
+
+extern WamWord pl_buff_signal_reg[];
+
 extern char *pl_reg_tbl[];
 
 #endif
 
-#define Init_Reg_Bank(x)
+#define Init_Reg_Bank(x)  save_reg_bank = pl_reg_bank = x
 
 
 #define Reg(i)			(((i)==0) ? (WamWord) TR 	: \
@@ -1091,23 +1109,43 @@ extern char *pl_reg_tbl[];
 
 #define Save_Machine_Regs(buff_save) \
   do { \
+    buff_save[0] = (WamWord) pl_reg_bank; \
+    buff_save[1] = (WamWord) TR; \
+    buff_save[2] = (WamWord) B; \
+    buff_save[3] = (WamWord) H; \
   } while(0)
 
 
 #define Restore_Machine_Regs(buff_save) \
   do { \
+    pl_reg_bank = (WamWordP) buff_save[0]; \
+    TR = (WamWordP) buff_save[1]; \
+    B = (WamWordP) buff_save[2]; \
+    H = (WamWordP) buff_save[3]; \
   } while(0)
 
 
 
 
-#define Start_Protect_Regs_For_Signal
+#define Start_Protect_Regs_For_Signal \
+  do { \
+    Save_Machine_Regs(pl_buff_signal_reg); \
+    pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS] = 1; \
+  } while(0)
 
 
-#define Stop_Protect_Regs_For_Signal
+#define Stop_Protect_Regs_For_Signal \
+  pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS] = 0; \
 
 
-#define Restore_Protect_Regs_For_Signal
+#define Restore_Protect_Regs_For_Signal \
+  do { \
+    if (pl_buff_signal_reg[NB_OF_USED_MACHINE_REGS]) { \
+      Restore_Machine_Regs(pl_buff_signal_reg); \
+      Stop_Protect_Regs_For_Signal; \
+    } \
+    pl_reg_bank = save_reg_bank; \
+  } while(0)
 
 
    /*--- End Register Generation ---*/
@@ -1126,21 +1164,21 @@ extern char *pl_reg_tbl[];
    /*--- Begin Tag Generation ---*/
 
 #define TAG_SIZE     		3
-#define TAG_SIZE_LOW 		2
-#define TAG_SIZE_HIGH		1
-#define VALUE_SIZE   		29
-#define TAG_MASK     		(PlULong)0x80000003
-#define VALUE_MASK   		(PlULong)0x7ffffffc
+#define TAG_SIZE_LOW 		3
+#define TAG_SIZE_HIGH		0
+#define VALUE_SIZE   		61
+#define TAG_MASK     		(PlULong)0x7
+#define VALUE_MASK   		(PlULong)0xfffffffffffffff8
 #define Tag_Mask_Of(w)		((PlLong) (w) & (TAG_MASK))
-#define Tag_From_Tag_Mask(w) 	(((PlULong) (w) >> 29) | ((w) & 3))
-#define Tag_Of(w)     		((((PlULong) (w) >> 31) << 2) | ((w) & 3))
+#define Tag_From_Tag_Mask(w) 	(w)
+#define Tag_Of(w)     		Tag_Mask_Of(w)
 #define TAG_REF_MASK		(PlULong)0
 #define TAG_LST_MASK		(PlULong)0x1
 #define TAG_STC_MASK		(PlULong)0x2
 #define TAG_ATM_MASK		(PlULong)0x3
-#define TAG_FLT_MASK		(PlULong)0x80000000
-#define TAG_FDV_MASK		(PlULong)0x80000001
-#define TAG_INT_MASK		(PlULong)0x80000003
+#define TAG_FLT_MASK		(PlULong)0x4
+#define TAG_FDV_MASK		(PlULong)0x5
+#define TAG_INT_MASK		(PlULong)0x7
 
 #define NB_OF_TAGS       	7
 #define REF        		0 
@@ -1153,11 +1191,11 @@ extern char *pl_reg_tbl[];
 
 	/* General Tag/UnTag macros */
 
-#define Tag_Long_Int(tm, v)  	((((PlLong) ((v) << 3)) >> 1) | (tm))
-#define Tag_Short_Uns(tm, v)	(((PlLong) (v) << 2) + (tm))
+#define Tag_Long_Int(tm, v)  	((((PlLong) ((v) << 3)) >> 0) | (tm))
+#define Tag_Short_Uns(tm, v)	(((PlLong) (v) << 3) + (tm))
 #define Tag_Address(tm, v)  	((PlLong) (v) + (tm))
 
-#define UnTag_Long_Int(w)    	((PlLong) ((w) << 1) >> 3)
+#define UnTag_Long_Int(w)    	((PlLong) ((w) << 0) >> 3)
 #define UnTag_Short_Uns(w)	UnTag_Long_Int(w)
 #define UnTag_Address(w)  	((WamWord *) ((w) & VALUE_MASK))
 
@@ -1171,12 +1209,12 @@ extern char *pl_reg_tbl[];
 #define Tag_ATM(v)  		Tag_Short_Uns(TAG_ATM_MASK, v)
 #define Tag_FLT(v)  		Tag_Address(TAG_FLT_MASK, v)
 #define Tag_FDV(v)  		Tag_Address(TAG_FDV_MASK, v)
-#define Tag_INT(v)  		(((PlULong) (v) << 2) | TAG_MASK)
+#define Tag_INT(v)  		(((PlULong) (v) << 3) | TAG_MASK)
 
 #define UnTag_REF(w)  		((WamWord *) (w))
 #define UnTag_LST(w)  		UnTag_Address(w)
 #define UnTag_STC(w)  		UnTag_Address(w)
-#define UnTag_ATM(w)  		((PlULong) (w) >> 2)
+#define UnTag_ATM(w)  		((PlULong) (w) >> 3)
 #define UnTag_FLT(w)  		UnTag_Address(w)
 #define UnTag_FDV(w)  		UnTag_Address(w)
 #define UnTag_INT(w)  		UnTag_Long_Int(w)
@@ -1213,9 +1251,9 @@ InfTag pl_tag_tbl[] =
   { "LST", ADDRESS, 1, 1},
   { "STC", ADDRESS, 2, 2},
   { "ATM", SHORT_UNS, 3, 3},
-  { "FLT", ADDRESS, 4, 80000000},
-  { "FDV", ADDRESS, 5, 80000001},
-  { "INT", LONG_INT, 7, 80000003}
+  { "FLT", ADDRESS, 4, 4},
+  { "FDV", ADDRESS, 5, 5},
+  { "INT", LONG_INT, 7, 7}
 };
 
 #else
@@ -1296,10 +1334,10 @@ PlLong pl_fixed_sizes;
 
 InfStack pl_stk_tbl[] =
 {
- { "trail", "Trail Stack (undo)", "TRAILSZ", &pl_def_trail_size, 4194304, 0, NULL },
- { "cstr", "Cstr Stack (constraints)", "CSTRSZ", &pl_def_cstr_size, 4194304, 0, NULL },
- { "global", "Global Stack (heap)", "GLOBALSZ", &pl_def_global_size, 8388608, 0, NULL },
- { "local", "Local Stack (control)", "LOCALSZ", &pl_def_local_size, 4194304, 0, NULL }
+ { "trail", "Trail Stack (undo)", "TRAILSZ", &pl_def_trail_size, 2097152, 0, NULL },
+ { "cstr", "Cstr Stack (constraints)", "CSTRSZ", &pl_def_cstr_size, 2097152, 0, NULL },
+ { "global", "Global Stack (heap)", "GLOBALSZ", &pl_def_global_size, 4194304, 0, NULL },
+ { "local", "Local Stack (control)", "LOCALSZ", &pl_def_local_size, 2097152, 0, NULL }
 };
 
 #else
@@ -1328,7 +1366,7 @@ extern InfStack pl_stk_tbl[];
  * Descr.: general header file                                             *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1366,7 +1404,7 @@ extern InfStack pl_stk_tbl[];
  * Descr.: FD Range Implementation - header file                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1404,7 +1442,7 @@ extern InfStack pl_stk_tbl[];
  * Descr.: boolean type definition - header file                           *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1501,7 +1539,7 @@ Range;
  * Descr.: FD Range Implementation - customizable header file              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1770,7 +1808,7 @@ char *Pl_Range_To_String(Range *range);
  * Descr.: FD instruction implementation - header file                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -1905,7 +1943,7 @@ char *Pl_Range_To_String(Range *range);
 
 #define CONSTRAINT_FRAME_SIZE      3
 
-#define OFFSET_OF_OPTIM_POINTER    1	/* this offset must corresponds to >>> */
+#define OFFSET_OF_OPTIM_POINTER    1	/* this offset must correspond to >>> */
 
 #define AF_Pointer(cf)             (*(WamWord **)    &(cf[0]))
 #define Optim_Pointer(cf)          (*(PlULong **)    &(cf[1]))	/* >>> this cell */
@@ -1991,7 +2029,7 @@ void Pl_Fd_Add_List_Dependency(WamWord *array, int chain_nb, WamWord *CF);
 
 void Pl_Fd_Before_Add_Cstr(void);
 
-Bool Pl_Fd_After_Add_Cstr(void);
+Bool Pl_Fd_After_Add_Cstr(Bool result_of_tell);
 
 void Pl_Fd_Stop_Constraint(WamWord *CF);
 
@@ -2271,11 +2309,8 @@ char *Pl_Fd_Variable_To_String0(WamWord *fdv_adr);
 
 
 #define fd_after_add_constraint			\
-  if (!Pl_Fd_After_Add_Cstr())			\
-    {						\
-      ret_val = FALSE;				\
-      goto lab_exit;				\
-    }
+  ret_val = Pl_Fd_After_Add_Cstr(ret_val);	 /* always followed by fd_return */
+
 
 
 
