@@ -6,7 +6,7 @@
  * Descr.: GNU Prolog - general header file (for users)                    *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2013 Daniel Diaz                                     *
+ * Copyright (C) 1999-2015 Daniel Diaz                                     *
  *                                                                         *
  * This file is part of GNU Prolog                                         *
  *                                                                         *
@@ -70,17 +70,17 @@ extern "C" {
 #define __GNU_PROLOG__        	1
 #define __GPROLOG__           	1
 #define __GPROLOG_MINOR__     	4
-#define __GPROLOG_PATCHLEVEL__	4
-#define __GPROLOG_VERSION__   	10404
+#define __GPROLOG_PATCHLEVEL__	5
+#define __GPROLOG_VERSION__   	10405
 
 #define PROLOG_DIALECT  	"gprolog"
 #define PROLOG_NAME     	"GNU Prolog"
-#define PROLOG_VERSION  	"1.4.4"
-#define PROLOG_DATE     	"Mar 27 2018"
-#define PROLOG_COPYRIGHT	"Copyright (C) 1999-2013 Daniel Diaz"
+#define PROLOG_VERSION  	"1.4.5"
+#define PROLOG_DATE     	"May  8 2020"
+#define PROLOG_COPYRIGHT	"Copyright (C) 1999-2020 Daniel Diaz"
 
-#define PL_MIN_INTEGER		-268435456
-#define PL_MAX_INTEGER		268435455
+#define PL_MIN_INTEGER		-1152921504606846976
+#define PL_MAX_INTEGER		1152921504606846975
 
 #define PL_REF        		0 
 #define PL_LST        		1 
